==> pitaya-framework-0.0.2.tgz: package/src/decorators.ts <==
import {
	IAppConfig,
	IAppViewDecorator,
	IAppViewDecoratorConfig,
	IDrawerConfig,
	IOnlyIfDecoratorFunction,
	IRoutesConfig,
	IRoutesConfigRoute,
	IViewDecorator,
	IViewDecoratorConfig
}                                 from "../index";
import { PitayaApp }              from "./app";
import { PitayaAppConfiguration } from "./app-configuration";
import { log }                    from "./logger";

interface IOfflineFunction extends Function
{
	offlineCallbackName: string;
}

const viewFunctionSource = {
	drawer : {
		items( ...items: string[] )
		{
			this._settings.drawerItems = items;
		},

		activeItem( name: string )
		{
			this._settings.drawerActiveItem = name;
		},

		disableItems( ...items: string[] )
		{
			this._settings.drawerItemsDisabled = items;
		},
	},
	public( isPublic: boolean = true )
	{
		this._settings.isPublic = isPublic;
	},
	theme( color: string )
	{
		this._settings.theme = color;
	}
};

const appFunctionSource = {
	title( titleName: string )
	{
		this._settings.title = titleName;
	},
	routesFile( filePath: string )
	{
		this._settings.routesConfigFile = filePath;
	},
	settingsFile( filePath: string )
	{
		this._settings.settingsFile = filePath;
	},
	settings( options: string | IAppConfig )
	{
		this._settings.configuration = options;
	},
	routes( routes: string | IRoutesConfigRoute )
	{
		this._settings.routes = routes;
	}
};

const findSourceFunctions = function( configParent, objectParent ): Map<Function, any[]>
{
	let functionCollection = new Map();

	for( const key of Object.keys( configParent ) )
	{
		const property = configParent[ key ];

		if( typeof property !== "object" || property instanceof Array )
		{
			functionCollection.set( objectParent[ key ], [ property ] );

			continue;
		}

		functionCollection = new Map( [
			...functionCollection,
			...findSourceFunctions( property, objectParent[ key ] )
		] );
	}

	return functionCollection;
};

/**
 * @Decorator
 * Executes the decorated method only if the condition is met
 * @param callback {Function}
 * @param elseCallbackName {string} (optional)
 */
export const onlyIf: IOnlyIfDecoratorFunction = function( callback: ( instance, method: string ) => boolean, elseCallbackName?: string )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue     = descriptor.value;
		const sElseCallback = elseCallbackName || propertyKey + "Else";

		descriptor.value = function()
		{
			/* Condition not fulfilled */
			if( !callback.call( this, this, propertyKey ) )
			{
				this[ sElseCallback ] && this[ sElseCallback ].call( this );

				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Raises an error and aborts execution if one of the given parameters is not defined in the decorated method
 * @param args {Array<string>}
 */
onlyIf.hasParameters = function( ...args: string[] )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue    = descriptor.value;
		const sValue       = fOldValue.toLocaleString();
		const aParamsNames = sValue.substring( sValue.indexOf( "(" ) + 1, sValue.indexOf( ")" ) ).split( ", " );
		const params       = {};

		/* Fill params with key<string>:values<Integer> */
		args.forEach( ( sParamName: string ) =>
		{
			params[ sParamName ] = aParamsNames.indexOf( sParamName );
		} );

		/* If method has none of the specified parameters */
		if( !Object.keys( params ).length )
		{
			log.warn( `{${target.constructor.name}} ${propertyKey}()`, `No matching parameter found. Decorator will have no effect` );
		}

		descriptor.value = function()
		{
			let bAbort = false;

			for( const key in params )
			{
				if( arguments[ params[ key ] ] !== undefined )
				{
					continue;
				}

				log.error( `{${target.constructor.name}} ${propertyKey}()`, `Parameter '${key}' has not been defined` );

				bAbort = true;
			}

			/* If errors were raised */
			if( bAbort )
			{
				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Raises an error and aborts execution if one of the given parameters is not defined in the object instancce of the decorated method
 * @param args {Array<string>}
 */
onlyIf.hasProperties = function( ...args: string[] )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue = descriptor.value;

		descriptor.value = function()
		{
			let bAbort = false;

			args.forEach( ( arg ) =>
			{
				if( this[ arg ] !== undefined )
				{
					return;
				}

				log.error( `{${target.constructor.name}} ${propertyKey}()`, `'${arg}' is not defined` );

				bAbort = true;
			} );

			/* If errors were raised */
			if( bAbort )
			{
				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Invokes the decorated function only if the application is online.
 * Optionally a callback can be invoked if defined. By default it´s name is [name of decorated function] + "Offline".
 */
onlyIf.isOnline = function( target, propertyKey: string, descriptor: PropertyDescriptor & IOfflineFunction ): any
{
	const fOldValue    = descriptor.value;
	const elseCallback = descriptor.offlineCallbackName || `${propertyKey}Offline`;

	descriptor.value = function()
	{
		/* Condition not fulfilled */
		if( !navigator.onLine )
		{
			this[ elseCallback ] && this[ elseCallback ].call( this );

			return;
		}

		return fOldValue.apply( this, arguments );
	};

	return descriptor;
};

/**
 * @Decorator
 * Invokes the decorated function only if the Development-Setting enabled.
 */
onlyIf.underDevelopment = function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
{
	const fOldValue = descriptor.value;

	descriptor.value = function()
	{
		if( !PitayaAppConfiguration.Development.enabled )
		{
			return;
		}

		return fOldValue.apply( this );
	};

	return descriptor;
};

// @ts-ignore
export const view: IViewDecorator = function( config: IViewDecoratorConfig )
{
	const functionsMap = findSourceFunctions( config, viewFunctionSource );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor()
			{
				super();

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

// @ts-ignore
view.drawer = function( config: IDrawerConfig )
{
	const functionsMap = findSourceFunctions( config, viewFunctionSource.drawer );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

view.drawer.items = function( ...items: string[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.items.call( this, ...items );
			}
		};
	};
};

view.drawer.disableItems = function( ...items: string[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.disableItems.call( this, ...items );
			}
		};
	};
};

view.drawer.activeItem = function( name: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.activeItem.call( this, name );
			}
		};
	};
};

/**
 * @Decorator
 * Redirects to a given route if the condition is met
 * @param route {string}
 * @param condition {Function}
 */
// @ts-ignore
view.redirectTo = function( route: string, condition: ( instance, route ) => boolean )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				condition.call( this, this, route || "" ) && PitayaApp.instance.navigate( route || "" );

				return;
			}
		};
	};
};

view.public = function( constructor: any ): any
{
	return class extends constructor
	{
		constructor( ...args )
		{
			super( ...args );

			viewFunctionSource.public.call( this, true );
		}
	};
};

view.theme = function( color: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.theme.call( this, color );
			}
		};
	};
};

// @ts-ignore
export const app: IAppViewDecorator = function( config: IAppViewDecoratorConfig )
{
	const functionsMap = findSourceFunctions( config, appFunctionSource );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor()
			{
				super();

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

app.title = function( titleName: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.title.call( this, titleName );
			}
		};
	};
};

/**
 * Configures the app
 * @param options {string | IAppConfig}: Can be either a configuration object or a path to a json file
 */
app.configure = function( options: string | IAppConfig )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.settings.call( this, options );
			}
		};
	};
};

/**
 * Configures the routes
 * @param routes {string | IRoutesConfig}: Can be either a configuration object or a path to a json file
 */
app.routes = function( routes: string | IRoutesConfigRoute[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.routes.call( this, routes );
			}
		};
	};
};

==> pitaya-framework-0.0.6.tgz: package/src/decorators.ts <==
import {
	IAppConfig,
	IAppViewDecorator,
	IAppViewDecoratorConfig,
	IDrawerConfig,
	IOnlyIfDecoratorFunction,
	IRoutesConfig,
	IRoutesConfigRoute,
	IViewDecorator,
	IViewDecoratorConfig
}                                 from "../interfaces.d";
import { PitayaApp }              from "./app";
import { PitayaAppConfiguration } from "./app-configuration";
import { log }                    from "./logger";

interface IOfflineFunction extends Function
{
	offlineCallbackName: string;
}

const viewFunctionSource = {
	drawer : {
		items( ...items: string[] )
		{
			this._settings.drawerItems = items;
		},

		activeItem( name: string )
		{
			this._settings.drawerActiveItem = name;
		},

		disableItems( ...items: string[] )
		{
			this._settings.drawerItemsDisabled = items;
		},
	},
	public( isPublic: boolean = true )
	{
		this._settings.isPublic = isPublic;
	},
	theme( color: string )
	{
		this._settings.theme = color;
	}
};

const appFunctionSource = {
	title( titleName: string )
	{
		this._settings.title = titleName;
	},
	routesFile( filePath: string )
	{
		this._settings.routesConfigFile = filePath;
	},
	settingsFile( filePath: string )
	{
		this._settings.settingsFile = filePath;
	},
	settings( options: string | IAppConfig )
	{
		this._settings.configuration = options;
	},
	routes( routes: string | IRoutesConfigRoute )
	{
		this._settings.routes = routes;
	}
};

const findSourceFunctions = function( configParent, objectParent ): Map<Function, any[]>
{
	let functionCollection = new Map();

	for( const key of Object.keys( configParent ) )
	{
		const property = configParent[ key ];

		if( typeof property !== "object" || property instanceof Array )
		{
			functionCollection.set( objectParent[ key ], [ property ] );

			continue;
		}

		functionCollection = new Map( [
			...functionCollection,
			...findSourceFunctions( property, objectParent[ key ] )
		] );
	}

	return functionCollection;
};

/**
 * @Decorator
 * Executes the decorated method only if the condition is met
 * @param callback {Function}
 * @param elseCallbackName {string} (optional)
 */
export const onlyIf: IOnlyIfDecoratorFunction = function( callback: ( instance, method: string ) => boolean, elseCallbackName?: string )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue     = descriptor.value;
		const sElseCallback = elseCallbackName || propertyKey + "Else";

		descriptor.value = function()
		{
			/* Condition not fulfilled */
			if( !callback.call( this, this, propertyKey ) )
			{
				this[ sElseCallback ] && this[ sElseCallback ].call( this );

				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Raises an error and aborts execution if one of the given parameters is not defined in the decorated method
 * @param args {Array<string>}
 */
onlyIf.hasParameters = function( ...args: string[] )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue    = descriptor.value;
		const sValue       = fOldValue.toLocaleString();
		const aParamsNames = sValue.substring( sValue.indexOf( "(" ) + 1, sValue.indexOf( ")" ) ).split( ", " );
		const params       = {};

		/* Fill params with key<string>:values<Integer> */
		args.forEach( ( sParamName: string ) =>
		{
			params[ sParamName ] = aParamsNames.indexOf( sParamName );
		} );

		/* If method has none of the specified parameters */
		if( !Object.keys( params ).length )
		{
			log.warn( `{${target.constructor.name}} ${propertyKey}()`, `No matching parameter found. Decorator will have no effect` );
		}

		descriptor.value = function()
		{
			let bAbort = false;

			for( const key in params )
			{
				if( arguments[ params[ key ] ] !== undefined )
				{
					continue;
				}

				log.error( `{${target.constructor.name}} ${propertyKey}()`, `Parameter '${key}' has not been defined` );

				bAbort = true;
			}

			/* If errors were raised */
			if( bAbort )
			{
				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Raises an error and aborts execution if one of the given parameters is not defined in the object instancce of the decorated method
 * @param args {Array<string>}
 */
onlyIf.hasProperties = function( ...args: string[] )
{
	return function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
	{
		const fOldValue = descriptor.value;

		descriptor.value = function()
		{
			let bAbort = false;

			args.forEach( ( arg ) =>
			{
				if( this[ arg ] !== undefined )
				{
					return;
				}

				log.error( `{${target.constructor.name}} ${propertyKey}()`, `'${arg}' is not defined` );

				bAbort = true;
			} );

			/* If errors were raised */
			if( bAbort )
			{
				return;
			}

			return fOldValue.apply( this, arguments );
		};

		return descriptor;
	};
};

/**
 * @Decorator
 * Invokes the decorated function only if the application is online.
 * Optionally a callback can be invoked if defined. By default it´s name is [name of decorated function] + "Offline".
 */
onlyIf.isOnline = function( target, propertyKey: string, descriptor: PropertyDescriptor & IOfflineFunction ): any
{
	const fOldValue    = descriptor.value;
	const elseCallback = descriptor.offlineCallbackName || `${propertyKey}Offline`;

	descriptor.value = function()
	{
		/* Condition not fulfilled */
		if( !navigator.onLine )
		{
			this[ elseCallback ] && this[ elseCallback ].call( this );

			return;
		}

		return fOldValue.apply( this, arguments );
	};

	return descriptor;
};

/**
 * @Decorator
 * Invokes the decorated function only if the Development-Setting enabled.
 */
onlyIf.underDevelopment = function( target, propertyKey: string, descriptor: PropertyDescriptor ): any
{
	const fOldValue = descriptor.value;

	descriptor.value = function()
	{
		if( !PitayaAppConfiguration.Development.enabled )
		{
			return;
		}

		return fOldValue.apply( this );
	};

	return descriptor;
};

// @ts-ignore
export const view: IViewDecorator = function( config: IViewDecoratorConfig )
{
	const functionsMap = findSourceFunctions( config, viewFunctionSource );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor()
			{
				super();

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

// @ts-ignore
view.drawer = function( config: IDrawerConfig )
{
	const functionsMap = findSourceFunctions( config, viewFunctionSource.drawer );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

view.drawer.items = function( ...items: string[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.items.call( this, ...items );
			}
		};
	};
};

view.drawer.disableItems = function( ...items: string[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.disableItems.call( this, ...items );
			}
		};
	};
};

view.drawer.activeItem = function( name: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.drawer.activeItem.call( this, name );
			}
		};
	};
};

/**
 * @Decorator
 * Redirects to a given route if the condition is met
 * @param route {string}
 * @param condition {Function}
 */
// @ts-ignore
view.redirectTo = function( route: string, condition: ( instance, route ) => boolean )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				condition.call( this, this, route || "" ) && PitayaApp.instance.navigate( route || "" );

				return;
			}
		};
	};
};

view.public = function( constructor: any ): any
{
	return class extends constructor
	{
		constructor( ...args )
		{
			super( ...args );

			viewFunctionSource.public.call( this, true );
		}
	};
};

view.theme = function( color: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				viewFunctionSource.theme.call( this, color );
			}
		};
	};
};

// @ts-ignore
export const app: IAppViewDecorator = function( config: IAppViewDecoratorConfig )
{
	const functionsMap = findSourceFunctions( config, appFunctionSource );

	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor()
			{
				super();

				for( const [ decorator, paramsCollection ] of functionsMap )
				{
					decorator.call( this, ...paramsCollection.flat() );
				}
			}
		};
	};
};

app.title = function( titleName: string )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.title.call( this, titleName );
			}
		};
	};
};

/**
 * Configures the app
 * @param options {string | IAppConfig}: Can be either a configuration object or a path to a json file
 */
app.configure = function( options: string | IAppConfig )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.settings.call( this, options );
			}
		};
	};
};

/**
 * Configures the routes
 * @param routes {string | IRoutesConfig}: Can be either a configuration object or a path to a json file
 */
app.routes = function( routes: string | IRoutesConfigRoute[] )
{
	return function( constructor: any ): any
	{
		return class extends constructor
		{
			constructor( ...args )
			{
				super( ...args );

				appFunctionSource.routes.call( this, routes );
			}
		};
	};
};
